//+------------------------------------------------------------------+
//|                                            ColorOptimization.mq5 |
//|                        Copyright 2017, MetaQuotes Software Corp. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright           "Copyright 2017, MetaQuotes Software Corp."
#property link                "https://www.mql5.com"
#property version             "1.00"
#property script_show_inputs

#include <ColorOptimization.mqh>

input string               ReportName     =  "ReportOptimizer-555849.xml"; // ReportOptimizer-555849
input string               OutputName     =  "ColorOptimization.htm";
input EOptimizatrionFactor Factor1        =  Profit;
input EOptimizatrionFactor Factor2        =  EquityDD_perc;
input EOptimizatrionFactor Factor3        =  RecoveryFactor;
input bool                 Factor1Invert  =  false;
input bool                 Factor2Invert  =  true;
input bool                 Factor3Invert  =  false;
input bool                 Sort           =  true;


//+------------------------------------------------------------------+
//| Script program start function                                    |
//+------------------------------------------------------------------+
void OnStart(){
   
   SOptimization2 opt;

   if(!OptimizerXMLReportToStruct2(ReportName,opt)){
      Alert("Error OptimizerXMLReportToStruct2");
      return;
   }
   
   SolveColorComponents(opt,Factor1,Factor2,Factor3,Factor1Invert,Factor2Invert,Factor3Invert);

   SolveSortFactor(opt);
   if(Sort){
      SortFactorSort(opt);
   }

   string report=HTMLStart("Color Optimization","style2.css")+TableContent(opt,Factor1,Factor2,Factor3)+Color2DPlanes(opt)+HTMLEnd();
      
   int h=FileOpen(OutputName,FILE_WRITE);
   if(h==-1){
      Alert("Error OutputName FileOpen");   
      return;
   }
   FileWriteString(h,report);
   FileClose(h);
   
   Alert("Ready!");
   
  }
//+------------------------------------------------------------------+

string HTMLStart(string aTitle,string aCSSFile="style.css"){
   string str="<!DOCTYPE html>\n";
   str=str+"<html>\n";
   str=str+"<head>\n";
   str=str+"<link href=\""+aCSSFile+"\" rel=\"stylesheet\" type=\"text/css\">\n";
   str=str+"<title>"+aTitle+"</title>\n";
   str=str+"</head>\n";  
   str=str+"<body>\n";     
   return str;
}

string HTMLEnd(){
   string str="</body>\n";
   str=str+"</html>\n";
   return str;
}